"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/**
 * word文件类
 */
var internal_1 = require("../internal");
var JSZip = require("jszip");
var assets_1 = require("../assets");
var sectionFileInfo_1 = tslib_1.__importDefault(require("./sectionFileInfo"));
var headerFileInfo_1 = tslib_1.__importDefault(require("./headerFileInfo"));
var footerFileInfo_1 = tslib_1.__importDefault(require("./footerFileInfo"));
var templates_1 = require("../templates");
var WordFile = /** @class */ (function () {
    function WordFile(name) {
        /**
         * 文件名
         */
        this.fileName = '';
        /**
         * 章节关联文件
         */
        this.rFileArr = [];
        /**
         * 页眉
         */
        this.headerFile = new headerFileInfo_1.default();
        /**
         * 页脚
         */
        this.footerFile = new footerFileInfo_1.default();
        this.fileName = name;
    }
    /**
     * 新增节点
     * @param html  要导出的word文本
     * @param options  页面配置项-支持边距和纵横向设置
     */
    WordFile.prototype.addSection = function (html, options) {
        if (options === void 0) { options = {}; }
        var rFile = new sectionFileInfo_1.default(this.rFileArr.length + 1, html, options, this);
        this.rFileArr.push(rFile);
    };
    /**
     *  为指定的节点设置页眉
     * @param html  页眉文本内容
     * @param sectionIdx 默认为所有节点设置页眉,大于-1之后
     */
    WordFile.prototype.setSectionHeader = function (text, exceptSectionIdx) {
        if (exceptSectionIdx === void 0) { exceptSectionIdx = -1; }
        if (!this.headerFile.id) {
            this.headerFile.id = 'rId-header';
            this.headerFile.target = "/word/" + this.headerFile.id + ".xml";
        }
        this.headerFile.htmlContent = text;
        if (exceptSectionIdx !== -1) {
            this.headerFile.exceptSectionIdx = exceptSectionIdx;
        }
    };
    /**
     *  为指定的节点设置页脚
     * @param html  页眉文本内容
     * @param sectionIdx 默认为所有节点设置页眉
     */
    WordFile.prototype.setSectionFooter = function (text, exceptSectionIdx) {
        if (exceptSectionIdx === void 0) { exceptSectionIdx = -1; }
        if (!this.footerFile.id) {
            this.footerFile.id = 'rId-footer';
            this.footerFile.target = "/word/" + this.footerFile.id + ".xml";
        }
        this.footerFile.htmlContent = text;
        if (exceptSectionIdx !== -1) {
            this.footerFile.exceptSectionIdx = exceptSectionIdx;
        }
    };
    /**
     * 生成word压缩包并返回异步字节流
     */
    WordFile.prototype.asBlob = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var zip, wordFolder;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        zip = new JSZip();
                        zip.file('[Content_Types].xml', internal_1.getBinaryData(this._generalContentTypesXml()), {
                            createFolders: false,
                        });
                        zip.folder('_rels').file('.rels', internal_1.getBinaryData(assets_1.relsXml), { createFolders: false });
                        wordFolder = zip.folder('word');
                        wordFolder.file('document.xml', this._renderDocumentFile(), {
                            createFolders: false,
                        });
                        this.rFileArr.forEach(function (rFile, idx) {
                            wordFolder.file(rFile.mhtName || 'afchunk.mht', rFile.generalMhtDocument(), {
                                createFolders: false,
                            });
                        });
                        // 写入页眉文件
                        if (this.headerFile.id) {
                            wordFolder.file(this.headerFile.mhtName || 'header.xml', templates_1.headerDocumentTemplate(this.headerFile.htmlContent), {
                                createFolders: false,
                            });
                            wordFolder.file('header-empty.xml', templates_1.emptyHeaderDocumentTemplate, {
                                createFolders: false,
                            });
                        }
                        // 写入页脚文件
                        if (this.footerFile.id) {
                            wordFolder.file(this.footerFile.mhtName || 'footer.xml', templates_1.footerDocumentTemplate(), {
                                createFolders: false,
                            });
                            wordFolder.file('footer-empty.xml', templates_1.emptyHeaderDocumentTemplate, {
                                createFolders: false,
                            });
                        }
                        wordFolder.folder('_rels')
                            .file('document.xml.rels', internal_1.getBinaryData(this._generalRelsXml()), {
                            createFolders: false,
                        });
                        return [4 /*yield*/, internal_1.generateDocument(zip)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * 生成文档结构
     * @private
     */
    WordFile.prototype._generalContentTypesXml = function () {
        var xmlStrArr = [];
        this.rFileArr.forEach(function (rFile, idx) {
            xmlStrArr.push(rFile.getContentTypeXml());
        });
        // 处理页眉
        if (this.headerFile.id) {
            xmlStrArr.push(this.headerFile.getContentTypeXml());
        }
        // 处理页角
        if (this.footerFile.id) {
            xmlStrArr.push(this.footerFile.getContentTypeXml());
        }
        return assets_1.contentTypesXmlHeader + xmlStrArr.join('\n') + '</Types>';
    };
    /**
     * 生成关系文档内容
     * @private
     */
    WordFile.prototype._generalRelsXml = function () {
        var relationArr = [];
        this.rFileArr.forEach(function (rFile) {
            relationArr.push(rFile.getRelationXml());
        });
        if (this.headerFile.id) {
            relationArr.push(this.headerFile.getRelationXml());
        }
        if (this.footerFile.id) {
            relationArr.push(this.footerFile.getRelationXml());
        }
        return assets_1.documentXmlRelsHeader + relationArr.join('\n') + '</Relationships>';
    };
    WordFile.prototype._renderDocumentFile = function () {
        var _this = this;
        var tmpHeader = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n  <w:document\n    xmlns:w=\"http://schemas.openxmlformats.org/wordprocessingml/2006/main\"\n    xmlns:m=\"http://schemas.openxmlformats.org/officeDocument/2006/math\"\n    xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\"\n    xmlns:wp=\"http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing\"\n    xmlns:a=\"http://schemas.openxmlformats.org/drawingml/2006/main\"\n    xmlns:ns6=\"http://schemas.openxmlformats.org/schemaLibrary/2006/main\"\n    xmlns:c=\"http://schemas.openxmlformats.org/drawingml/2006/chart\"\n    xmlns:ns8=\"http://schemas.openxmlformats.org/drawingml/2006/chartDrawing\"\n    xmlns:dgm=\"http://schemas.openxmlformats.org/drawingml/2006/diagram\"\n    xmlns:pic=\"http://schemas.openxmlformats.org/drawingml/2006/picture\"\n    xmlns:ns11=\"http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing\"\n    xmlns:dsp=\"http://schemas.microsoft.com/office/drawing/2008/diagram\"\n    xmlns:ns13=\"urn:schemas-microsoft-com:office:excel\"\n    xmlns:o=\"urn:schemas-microsoft-com:office:office\"\n    xmlns:v=\"urn:schemas-microsoft-com:vml\"\n    xmlns:w10=\"urn:schemas-microsoft-com:office:word\"\n    xmlns:ns17=\"urn:schemas-microsoft-com:office:powerpoint\"\n    xmlns:odx=\"http://opendope.org/xpaths\"\n    xmlns:odc=\"http://opendope.org/conditions\"\n    xmlns:odq=\"http://opendope.org/questions\"\n    xmlns:odi=\"http://opendope.org/components\"\n    xmlns:odgm=\"http://opendope.org/SmartArt/DataHierarchy\"\n    xmlns:ns24=\"http://schemas.openxmlformats.org/officeDocument/2006/bibliography\"\n    xmlns:ns25=\"http://schemas.openxmlformats.org/drawingml/2006/compatibility\"\n    xmlns:ns26=\"http://schemas.openxmlformats.org/drawingml/2006/lockedCanvas\">\n    <w:body>";
        var renderOptionArr = [];
        this.rFileArr.forEach(function (rFile, idx) {
            var isLastOne = (idx === _this.rFileArr.length - 1);
            renderOptionArr.push(rFile.documentOPENXML(isLastOne));
        });
        return tmpHeader + renderOptionArr.join('') + "</w:body>\n </w:document>";
    };
    return WordFile;
}());
exports.default = WordFile;
